/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

// QtFlickr.js
// Animated Flickr Photo Viewer

function tr(s) { return s; }

function FlickrWidget(parent)
{
    QWidget.call(this, parent);

	this.flickrUrl = "http://api.flickr.com/services/feeds/photos_public.gne?format=rss2"
	this.imageUrls = new Array();

	this.lineEdit = new QLineEdit(this);
	this.lineEdit["returnPressed"].connect(this, this.tagsUpdated);

	this.imageLabel = new QLabel(this);
	this.imageLabel.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored);
    this.imageLabel.alignment = Qt.AlignCenter;
    
    this.goButton = new QPushButton(tr("Go"),this);
    this.goButton["clicked"].connect(this, this.tagsUpdated);

	this.imagePixmap = new QPixmap();
	this.nextPixmap = new QPixmap();

	this.fetchImageTimer = new QTimer(this);
	this.fetchImageTimer["timeout"].connect(this, this.fetchImageTimerTimeout);
	
	this.rotTimer = new QTimer(this);
	this.rotTimer["timeout"].connect(this, this.rotTimerTimeout);
	
	this.speedSlider = new QSlider(Qt.Horizontal,this);
	this.speedSlider.minimum = 0;
	this.speedSlider.maximum = 4;
	this.speedSlider.value = 2;
	this.speedSlider["valueChanged(int)"].connect(this,this.speedChanged);
	
	this.rotationSpeedLabel = new QLabel(tr("Rotation speed (2):"));
	
	this.angle = 90;

	var hBoxLayout = new QHBoxLayout();
	hBoxLayout.addWidget(new QLabel(tr("Tags:"),this),0,0);
	hBoxLayout.addWidget(this.lineEdit,1,0);
	hBoxLayout.addWidget(this.goButton,0,0);
	
	this.layout = new QVBoxLayout(this);
	this.layout.addLayout(hBoxLayout);
	this.layout.addWidget(this.imageLabel,1,0);
	this.layout.addWidget(this.rotationSpeedLabel,0,0);
	this.layout.addWidget(this.speedSlider,0,0);

	/* Bind handler for keyPressEvent */
	FlickrWidget.prototype.keyPressEvent = keyDown;

    this.windowTitle = tr("QtFlickr: Flickr Photo Viewer");
	this.resize(300,300);
}

FlickrWidget.prototype = new QWidget();

/* Key event handler */
keyDown = function(event) {
	var key = event.key();

    if (key == Qt.Key_F6)
		this.toggleWindowState();
}

FlickrWidget.prototype.toggleWindowState = function() {
	var state = this.windowState() ^ Qt.WindowFullScreen;
	/* State is 4 when fullscreen, 0 in normal state */
	if(state != 4) {
		this.setWindowState(Qt.WindowNoState | !Qt.WindowFullScreen);
	}
	else {
		this.setWindowState(Qt.WindowFullScreen);
	}
}

FlickrWidget.prototype.tagsUpdated = function() {
	this.loadFeed();
}

FlickrWidget.prototype.fetchImageTimerTimeout = function() {
	this.showRandomImage();
}

FlickrWidget.prototype.rotTimerTimeout = function() {
	var speed = 0;
	
	this.angle = this.angle + (this.speedSlider.value * 5);
	
	if(this.angle > 450) {
		this.angle = 90;
	}
	else if((this.angle == 90) || (this.angle == 270)) {
		this.imagePixmap = new QPixmap(this.nextPixmap);
		//print("Change now! " + (this.speedSlider.value * 5));
	}
	//print(this.angle);
	
	var trans = new QTransform();
	trans.rotate(this.angle,Qt.YAxis);
	trans.rotate(this.angle,Qt.ZAxis);

	this.imageLabel.setPixmap(this.imagePixmap.transformed(trans,Qt.SmoothTransformation));
}

FlickrWidget.prototype.speedChanged = function(value) {
	this.rotationSpeedLabel.text = 'Rotation speed (' + value.toString() + '):';
}

FlickrWidget.prototype.showRandomImage = function() {
	if(this.imageUrls.length != 0) {
		this.fetchImageTimer.stop();
		var accessManager = new QNetworkAccessManager(this);
		accessManager["finished(QNetworkReply*)"].connect(this, this.imageReplyFinished);
		var urlString = this.imageUrls[Math.floor((this.imageUrls.length-2)*Math.random())];
		accessManager.get(new QNetworkRequest((new QUrl(urlString))));
	}
}

FlickrWidget.prototype.imageReplyFinished = function(reply) {
	var data = reply.readAll();
	this.nextPixmap.loadFromData(data, 0, Qt.AutoColor);
	if(!this.nextPixmap.isNull()) {
		this.nextPixmap = this.nextPixmap.scaled(this.imageLabel.width*0.8,this.imageLabel.height*0.8,Qt.KeepAspectRatio,Qt.SmoothTransformation);
		this.rotTimer.start(50);
	}
	else {
		/* imagePixmap was null, image data was not available (bad url)*/
		this.imageLabel.text = '<center>No image available</center>';
	}
	/* fetch next image after five seconds */
	this.fetchImageTimer.start();
}



FlickrWidget.prototype.loadFeed = function() {
	this.imageUrls = [];
	var url = this.flickrUrl + "&tags=" + this.lineEdit.text;
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, flickrReplyFinished);
	accessManager.get(new QNetworkRequest(new QUrl(url)));
	/* fetch next image after five seconds */
	this.fetchImageTimer.start(5000);
}

flickrReplyFinished = function(reply) {
	var xml = new QXmlStreamReader();
	xml.addData(reply.readAll());
	
	while (!xml.atEnd()) {
        xml.readNext();
        if(xml.isStartElement()) {
            if(xml.name() == "enclosure") {
            	this.imageUrls.push(xml.attributes().value("url").toString());
			}
		} 
    }
    if (xml.hasError() && (xml.error() != QXmlStreamReader.PrematureEndOfDocumentError)) {
        print("XML ERROR:", xml.lineNumber() + ":", xml.errorString());
    }
}

/* To use the application in standalone mode, remove the comment around this code  
var wdgt = new FlickrWidget();
wdgt.show();

QCoreApplication.exec();
*/