/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

// rss.qs
// RSSWidget for RSS-feed reading

function tr(s) { return s; }

function createIconReplyCallback(buttonName) {
	return new Function("reply",
		"var data = reply.readAll();"+
		"var pixmap = new QPixmap();"+
		"pixmap.loadFromData(data,0,Qt.AutoColor);"+
		"this."+buttonName+".icon = new QIcon(pixmap);"
	);
}

function RSSWidget(parent)
{
    QWidget.call(this, parent);

	// At first we create some variables
    this.xml = new QXmlStreamReader();
    this.url = "";
    this.currentTag = "";
    this.linkString = "";
    this.titleString = "";
    this.descriptionString = "";
    this.pubDateString = "";
    this.connectionId = -1;
	this.selectedItems = null;
	this.itemIndex = 0;
	
	this.RSS_FEEDS_FILENAME = "./QtRss_AutoOpenFeeds.csv";
	this.MAGIC_NUMBER = "22022009";

	// user can edit this.lineEdit and add RSSFeed
    this.lineEdit = new QLineEdit(this);

	// Create some icons for buttons
	/*
	addIcon = new QIcon("./images/add.ico");
	abortIcon = new QIcon("./images/abort.ico");
	reloadIcon = new QIcon("./images/reload.ico");
	removeIcon = new QIcon("./images/remove.ico");
	*/
	
	var accessManager = new QNetworkAccessManager(this);
	accessManager.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/add.ico")));
    accessManager.finished.connect(this, createIconReplyCallback("addButton"));
	
	var accessManager1 = new QNetworkAccessManager(this);
	accessManager1.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/abort.ico")));
    accessManager1.finished.connect(this, createIconReplyCallback("abortButton"));
	
	var accessManager2 = new QNetworkAccessManager(this);
	accessManager2.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/reload.ico")));
    accessManager2.finished.connect(this, createIconReplyCallback("reloadButton"));
	
	var accessManager3 = new QNetworkAccessManager(this);
	accessManager3.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/remove.ico")));
    accessManager3.finished.connect(this, createIconReplyCallback("removeButton"));

	// Add four buttons
    this.addButton = new QPushButton(tr("Add"), this);
    this.addButton.enabled = false;
    this.abortButton = new QPushButton(tr("Abort"), this);
    this.abortButton.enabled = false;
    this.removeButton = new QPushButton(tr("Remove"), this);
    this.removeButton.enabled = false;
    this.reloadButton = new QPushButton(tr("Reload"), this);
    this.reloadButton.enabled = false;

	// Add tree widget and set connections for it
    this.channelTreeWidget = new QTreeWidget(this);
    this.channelTreeWidget["itemDoubleClicked(QTreeWidgetItem*, int)"].connect(
        this, this.itemDoubleClicked);
    this.channelTreeWidget["itemSelectionChanged ()"].connect(
        this, this.itemSelectionChanged);
    this.channelTreeWidget["itemClicked(QTreeWidgetItem*, int)"].connect(
        this, this.itemClicked);
    this.channelTreeWidget["itemActivated(QTreeWidgetItem*, int)"].connect(
        this, this.itemClicked);
        
    // Header labels are shown above treeWidget
    var headerLabels = new Array();
    headerLabels.push(tr("Title"));
    //headerLabels.push(tr("Link")); // commented our to simplify ui
    //headerLabels.push(tr("Description")); // commented out to simplify ui
    this.channelTreeWidget.setHeaderLabels(headerLabels);
    this.channelTreeWidget.header().setResizeMode(QHeaderView.Stretch);

	this.descriptionText = new QTextBrowser(this);
	
    this.http = new QHttp(this);
    this.http.readyRead.connect(this, this.readData);
    this.http.requestFinished.connect(this, this.finished);

    this.lineEdit["returnPressed"].connect(this, this.add);
    this.lineEdit["textChanged(QString)"].connect(
    	this, this.lineEditChanged);
    	
    this.addButton["clicked"].connect(this, this.add);
    this.abortButton["clicked"].connect(this.http, this.http.abort);
    this.removeButton["clicked"].connect(this, this.remove);
    this.reloadButton["clicked"].connect(this, this.reload);

    var layout = new QVBoxLayout(this);
	var vSplitter = new QSplitter(Qt.Vertical, this);
    var hboxLayout = new QHBoxLayout();

    // ### working around problem with addWidget() binding
	hboxLayout.addWidget(new QLabel(tr("RSS Feed URL:")),0,0);
    hboxLayout.addWidget(this.lineEdit, 0, 0);
    hboxLayout.addWidget(this.addButton, 0, 0);
    hboxLayout.addWidget(this.abortButton, 0, 0);
    hboxLayout.addWidget(this.removeButton, 0, 0);
    hboxLayout.addWidget(this.reloadButton, 0, 0);

    vSplitter.addWidget(this.channelTreeWidget);
    vSplitter.addWidget(this.descriptionText);

    layout.addLayout(hboxLayout);
    layout.addWidget(vSplitter,0,0);

    this.windowTitle = tr("RSSWidget");
    this.resize(800,480);  
}

RSSWidget.prototype = new QWidget();

/* Function: add some feeds for demo purposes */
RSSWidget.prototype.addDemoContent = function() {
    // Tell everybody why it takes so long
    this.descriptionText.setHtml("<strong>Adding demo content...</strong><br>Please wait.");
    this.add("http://newsrss.bbc.co.uk/rss/newsonline_world_edition/front_page/rss.xml");
    // wait until we can add more feeds
    while(this.lineEdit.readOnly);
    this.add("http://rss.cnn.com/rss/cnn_world.rss");
    while(this.lineEdit.readOnly);
    this.add("http://feeds.digg.com/digg/container/technology/popular.rss");
    while(this.lineEdit.readOnly);
    this.descriptionText.setHtml("");
}

/* Function: add RSS Feed from the url. Url is a string. */
RSSWidget.prototype.add = function(url)
{
	// Deactivate and activate parts of user interface
    this.lineEdit.readOnly = true;
    this.addButton.enabled = false;
    this.abortButton.enabled = true;
    this.xml.clear();

	// Copy url to this.url and use it if it is not empty.
	// if the url is empty use text from the lineEdit
	if(url == "")
		this.url = this.lineEdit.text;
	else
		this.url = url;
	
	// Check if url is empty after all
	if(this.url == "") {
		print("Error: url is empty.");
		this.lineEdit.readOnly = false;
		this.addButton.enabled = true;
		this.abortButton.enabled = false;
		return;
	}
	
	try {	
    	this.qurl = new QUrl(this.url);
    	this.http.setHost(this.qurl.host());
    	this.connectionId = this.http.get(this.qurl.path());
	}
	catch(err) {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: " + err;
		messageBox.show();
		this.lineEdit.readOnly = false;
		this.addButton.enabled = true;
		this.abortButton.enabled = false;
		return;
	}
}

/* Function: removes selected item */
RSSWidget.prototype.remove = function() {
	this.channelTreeWidget.takeTopLevelItem(
		this.channelTreeWidget.indexOfTopLevelItem(this.selectedItems[0]));
}

/* Function: Read Http data. If statusCode is 200 everything is ok.
 * else something has gone wrong. */
RSSWidget.prototype.readData = function(resp)
{
    if (resp.statusCode() != 200) {
        this.http.abort();
        this.itemIndex = 0;
	}
    else {
        this.xml.addData(this.http.readAll());
        // Parse this.xml
        this.parseXml();
    }
}

/* Function: Http connection is finished, let's reactivate our 
 * user interface */
RSSWidget.prototype.finished = function(id, error)
{
    if (error) {
        print("Received error during HTTP fetch."); // ### qWarning()
        this.lineEdit.readOnly = false;
        this.abortButton.enabled = false;
        this.addButton.enabled = true;
    } else if (id == this.connectionId) {
        this.lineEdit.readOnly = false;
        this.abortButton.enabled = false;
        this.addButton.enabled = true;
    }
}

/* Function: Use parseXml to parse this.xml data
 * adds treeWidgetItems to treeWidget */
RSSWidget.prototype.parseXml = function()
{
	var xmlElement;
	
    while (!this.xml.atEnd()) {
        this.xml.readNext();
        if (this.xml.isStartElement()) {
            if (this.xml.name() == "item") {
                this.linkString = this.xml.attributes().value("rss:about").toString();
                xmlElement = "item";
           	}
           	if (this.xml.name() == "channel") {
           		xmlElement = "channel";
           		this.channel = new QTreeWidgetItem();
			}
            this.currentTag = this.xml.name().toString();
        // We are at the end of an element
        } else if (this.xml.isEndElement()) {
        	if (xmlElement == "channel") {
        		if (this.xml.name() == "title") {
        			this.channel.setText(0,this.titleString);
        			this.titleString = "";
				}
				if (this.xml.name() == "link" ) {
					this.channel.setText(1,this.url);	
				}
				if (this.xml.name() == "description") {
					this.channel.setText(2, this.descriptionString);
					this.descriptionString = "";
				}
				if (this.xml.name() == "pubDate") {
					this.channel.setText(3, this.pubDateString);
					this.pubDateString = "";
				}
				this.linkString = "";
			}
			// if we have reached the end of the channel element
			// we can add our new topLevelItem to the treeWidget
			if (this.xml.name() == "channel") {
				this.channelTreeWidget.insertTopLevelItem(this.itemIndex, this.channel);
				// save current tree to external file
				this.itemIndex = 0;
			}

            if (this.xml.name() == "item") {
                var item = new QTreeWidgetItem(1); // item type is 1
                item.setText(0, this.titleString);
                item.setText(1, this.linkString);
                if(this.descriptionString.length > 0)
                	item.setText(2, this.descriptionString);
                if(this.pubDateString.length > 0)
                	item.setText(3, this.pubDateString);
                	
                // Add this item uder current channel
                this.channel.addChild(item);

				// Empty our variables
                this.titleString = "";
                this.linkString = "";
                this.descriptionString = "";
                this.pubDateString = "";
            }
        // Collecting items we would propably want to save in the tree
        } else if (this.xml.isCharacters() && !this.xml.isWhitespace()) {
            if (this.currentTag == "title")
                this.titleString += this.xml.text().toString();
            else if (this.currentTag == "link")
                this.linkString += this.xml.text().toString();
            else if (this.currentTag == "description")
            	this.descriptionString += this.xml.text().toString();
            else if (this.currentTag == "pubDate")
            	this.pubDateString += this.xml.text().toString();
        }
    }
    if (this.xml.hasError() && (this.xml.error() != QXmlStreamReader.PrematureEndOfDocumentError)) {
        print("XML ERROR:", this.xml.lineNumber() + ":", this.xml.errorString());
        this.http.abort();
    }
}

/* Function: when treeWidgetItem is doubleClicked
 * Open url related to the item. */
RSSWidget.prototype.itemDoubleClicked = function(item)
{
	// Url is found from the second column
	QDesktopServices.openUrl(new QUrl(item.text(1)));
}

/* Function: when treeWidgetItem is clicked once
 * Open item description to the descriptionText textBrowser */
RSSWidget.prototype.itemClicked = function(item)
{
	// Description text is found from the third column
	// fourth column contains pubDate property if available
	this.descriptionText.setHtml("<big>" + item.text(0) + "</big><br>" 
						+ "<small>" + item.text(3) + "</small><br>"
						+ "<small>(" + item.text(1) + ")</small>"
						+ "<hr>" + item.text(2));
}

/* Function: when treeWidgetItem item selectionChanged
 * Check if item is top level item and change user interface accordingly */
RSSWidget.prototype.itemSelectionChanged = function()
{
	//this.selectedItems = null;
	this.selectedItems = this.channelTreeWidget.selectedItems();
	
	// If selectedItems[0] type is not 1 then the item is top level item
	if(this.selectedItems[0] != null
		&& this.selectedItems[0].type() != 1) {
		this.removeButton.enabled = true;
		this.reloadButton.enabled = true;
	} else {
		this.removeButton.enabled = false;
		this.reloadButton.enabled = false;
	}
}

/* Function: when reloadButton is clicked
 * we will remove the item and reload it */
RSSWidget.prototype.reload = function () {
	// Remember itemIdex and reload to same position
	this.itemIndex = this.channelTreeWidget.indexOfTopLevelItem(this.selectedItems[0]);
	while(this.lineEdit.readOnly);
	this.url = this.selectedItems[0].text(1);
	this.remove(); // remove selected
	this.add(this.url);
}

/* Function: When lineEdit is changed check if addButton should still
 * be clickable */
RSSWidget.prototype.lineEditChanged = function(text)
{
	if(this.lineEdit.text.length > 0) {
		this.addButton.enabled = true;
	}
	else {
		this.addButton.enabled = false;
	}
}

RSSWidget.prototype.showEvent = function (event) {
	this.loadTree();
}

RSSWidget.prototype.closeEvent = function (event) {
	this.saveTree();
}

RSSWidget.prototype.saveTree = function()
{
	var filename = this.RSS_FEEDS_FILENAME;
	if(filename != "") {
		var file = new QFile(filename);
		if(file.exists()) {
			file.remove();
		}
		file.open(QIODevice.ReadWrite);
		var stream = new QTextStream(file);
		stream.writeString(this.MAGIC_NUMBER + "\n");
		for(i = 0; i < this.channelTreeWidget.topLevelItemCount; i=i+1) {
			var treeItem = this.channelTreeWidget.topLevelItem(i);
			stream.writeString(treeItem.text(1) + "\n");
		}
		file.close();
	}
}

RSSWidget.prototype.loadTree = function()
{
	var filename = this.RSS_FEEDS_FILENAME;
	if(filename != "") {
		var file = new QFile(filename);	
		if(file.exists()) {
			file.open(QIODevice.ReadOnly);
			var stream = new QTextStream(file);
			if(stream.readLine() != this.MAGIC_NUMBER) {
				var messageBox = new QMessageBox(this);
				messageBox.text = "Error: File is not valid.";
				messageBox.show();
				file.close();
				return;
			}
			this.channelTreeWidget.clear();
			while(!stream.atEnd()) {
				var line = stream.readLine();
    			this.add(line);
    			// wait until we can add more feeds
    			while(this.lineEdit.readOnly);
			}
			file.close();
		}
	}
}


/* To use the application in standalone mode, remove the comment around this code
var rsslisting = new RSSWidget();
rsslisting.show();

QCoreApplication.exec();
*/

