/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/
 
function tr(s) { return s; }

function ScrapBook(parent) {
	QWidget.call(this, parent);
	
	this.TABS_FILENAME = "QtScrapbook_AutoOpenURLs.ini";
	this.MAGIC_NUMBER = "25032009";

	this.acceptDrops = true;

	this.addNewTabButton = new QPushButton(tr("New Tab"), this);
	this.addNewTabButton["clicked"].connect(this, this.addNewTab);

	this.tabWidget = new QTabWidget(this);
	this.tabWidget["currentChanged(int)"].connect(this, this.tabChanged);
	this.tabWidget["tabCloseRequested(int)"].connect(this, this.tabCloseRequested);
	this.tabWidget.tabsClosable = true;
	this.tabWidget.elideMode = Qt.ElideRight;
	this.tabWidget.movable = true;
	this.tabWidget.setCornerWidget(this.addNewTabButton, Qt.TopRightCorner);
	//this.tabWidget.addTab(new dropImg(), "");
	
	this.slider = new QSlider(Qt.Horizontal,this);
	this.slider.minimum = 5;
	this.slider.maximum = 180;
	this.slider["valueChanged(int)"].connect(this,this.sliderValueChanged);
	this.sliderLabel = new QLabel(tr("5 sec."),this);
	this.changeTabsButton = new QPushButton(tr("Auto Change"), this);
	this.changeTabsButton.checkable = true;
	this.changeTabsButton["toggled"].connect(this, this.changeTabsButtonToggled);
	this.changeTabsTimer = new QTimer(this);
	this.changeTabsTimer["timeout"].connect(this, this.changeTabsTimerTimeout);
	
	var hboxLayout = new QHBoxLayout();
	hboxLayout.addWidget(this.changeTabsButton, 0,0);
	hboxLayout.addWidget(new QLabel(tr("Interval:")),0,0);
	hboxLayout.addWidget(this.slider,0,0);
	hboxLayout.addWidget(this.sliderLabel,0,0);
	hboxLayout.addStretch();
	
	this.layout = new QVBoxLayout(this);
	this.layout.addWidget(this.tabWidget,0,0);
	this.layout.addLayout(hboxLayout);
	
	this.resize(400,400);
	this.windowTitle = tr("ScarpbookWidget");
}

ScrapBook.prototype = new QWidget();

ScrapBook.prototype.addNewTab = function () {
	var wdgt = new dropImg()
	var index = this.tabWidget.addTab(wdgt,"");
	this.tabWidget.currentIndex = index;
}

ScrapBook.prototype.tabChanged = function () {
	if(this.changeTabsButton.checked == true) {
		this.changeTabsTimer.stop();
		this.changeTabsTimer.start(this.slider.value*1000);
	}
	this.windowTitle = "Webcam Widget - " + this.tabWidget.widget(this.tabWidget.currentIndex).windowTitle;
}

ScrapBook.prototype.tabCloseRequested = function (index) {
	this.closeTabButtonClicked();
}

ScrapBook.prototype.closeTabButtonClicked = function () {
	if(this.tabWidget.count >= 2) {
		this.tabWidget.removeTab(this.tabWidget.currentIndex);
	}
}

ScrapBook.prototype.changeTabsTimerTimeout = function () {
	if(this.tabWidget.currentIndex != this.tabWidget.count-1) {
		this.tabWidget.currentIndex = this.tabWidget.currentIndex +1;
	}
	else {
		this.tabWidget.currentIndex = 0;
	}
}

ScrapBook.prototype.sliderValueChanged = function (value) {
	this.sliderLabel.text = value.toString() + " sec.";
	if(this.changeTabsButton.checked == true) {
		this.changeTabsTimer.stop();
		this.changeTabsTimer.start(this.slider.value*1000);
	}
}

ScrapBook.prototype.changeTabsButtonToggled = function () {
	if(this.changeTabsButton.checked == true) {
		this.changeTabsTimer.start(this.slider.value*1000);
	}
	else {
		this.changeTabsTimer.stop();
	}
}

ScrapBook.prototype.dragEnterEvent = function (event) {
	if (event.mimeData().hasUrls())
		event.acceptProposedAction();
}

ScrapBook.prototype.dropEvent = function (event) {
	var wdgt = new dropImg()
	var index = this.tabWidget.addTab(wdgt,"");
	this.tabWidget.currentIndex = index;
	this.tabWidget.widget(index).event(event);
}

ScrapBook.prototype.closeEvent = function (event) {
	this.save();
}

ScrapBook.prototype.showEvent = function (event) {
	this.load();
}

ScrapBook.prototype.save = function()
{
	var filename = this.TABS_FILENAME;
	if(filename != "") {
		var file = new QFile(filename);
		if(file.exists()) {
			file.remove();
		}
		file.open(QIODevice.ReadWrite);
		var stream = new QTextStream(file);
		stream.writeString(this.MAGIC_NUMBER + "\n");
		stream.writeString(this.width + "\n");
		stream.writeString(this.height + "\n");
		for(i = 0; i < this.tabWidget.count; i=i+1) {
			var wdgt = this.tabWidget.widget(i);
			var url = wdgt.windowTitle;
			if(url != undefined) {
				stream.writeString(url + "\n");
			}
		}
		file.close();
	}
}

ScrapBook.prototype.load = function()
{
	var filename = this.TABS_FILENAME;
	if(filename != "") {
		var file = new QFile(filename);	
		if(file.exists()) {
			file.open(QIODevice.ReadOnly);
			var stream = new QTextStream(file);
			if(stream.readLine() != this.MAGIC_NUMBER) {
				var messageBox = new QMessageBox(this);
				messageBox.text = "Error: File is not valid. (" + this.TABS_FILENAME + ")";
				messageBox.show();
				file.close();
				return;
			}
			var width = stream.readLine();
			var height = stream.readLine();
			//this.resize(width,height);
			while(!stream.atEnd()) {
				var line = stream.readLine();
				var wdgt = new dropImg(line,this)
				this.tabWidget.addTab(wdgt,"");
			}
			file.close();
		}
	}
	if(this.tabWidget.count == 0) {
		this.tabWidget.addTab(new dropImg(),"");
	}
}


/*
	dropImg widget definition starts here
*/

function dropImg(loadUrl, parent)
{
    QWidget.call(this, parent);

	this.acceptDrops = true;

	this.img = new QImage;
	this.url = new QUrl;

	this.manager = new QNetworkAccessManager();
	this.manager["finished(QNetworkReply*)"].connect(this, this.handleNetworkData);

	this.timer = new QTimer;
	this.timer["timeout"].connect(this, this.timerTimeout);
	
	this.windowTitle = tr("undefined");

	if(loadUrl) {
		this.windowTitle = tr(loadUrl);
		this.url = new QUrl(loadUrl, QUrl.TolerantMode);
		this.manager.get(new QNetworkRequest(this.url));
		this.timer.start(20000);
	}
}

dropImg.prototype = new QWidget();

dropImg.prototype.getUrlString = function () {
	return this.urlString;
}

dropImg.prototype.timerTimeout = function () {
	this.manager.get(new QNetworkRequest(this.url));
}

dropImg.prototype.loadImage = function (image) {
	//var img = new QImage(image.convertToFormat(QImage.Format_ARGB32_Premultiplied));
    if (image.isNull()) {
        this.setWindowTitle("Can't load the image. Please drag and drop a new image.");
    } else {
        this.img = new QImage(image);
    }
    this.update();
}

dropImg.prototype.handleNetworkData = function (reply) {
	var data = reply.readAll();
	try {
		this.img.loadFromData(data, Qt.AutoColor);
	}
	catch (err) {
		print(err);
	}
	/*
	try {
		var code = data.toString();
		cade = code.replace("QCoreApplication.exec();","");
		eval(code);
	}
	catch(err) {
		print(err);
	}
	*/
	this.update();
}

dropImg.prototype.dragEnterEvent = function (event) {
	if (event.mimeData().hasUrls())
		event.acceptProposedAction();
}

dropImg.prototype.dropEvent = function (event) {
	var urls = event.mimeData().urls();
	if(urls.length) {
		try {
			this.url = new QUrl(urls, QUrl.TolerantMode);
			this.setWindowTitle(urls);
		}
		catch(err) {
			return;
		}
        //if (event.mimeData().hasImage()) {
        //    var img = new QImage(event.mimeData().imageData());
			//var filename = new QFileInfo(url.path()).fileName();
			//this.setWindowTitle(filename);
		//	this.loadImage(img);
		//}
		//else {
			this.manager.get(new QNetworkRequest(this.url));
		//}
		event.acceptProposedAction();
		this.timer.start(20000);
	}
}

/*
dropImg.prototype.mousePressEvent = function (event) {
	if (event.button() == Qt.LeftButton) {
        var drag = new QDrag(this);
        var mimeData = new QMimeData();
		mimeData.setUrls(this.url);
        drag.setMimeData(mimeData);
        drag.exec();
	}
}
*/

dropImg.prototype.paintEvent = function (event) {
	if(this.img.isNull()) {
		return;
	}
	
	var painter = new QPainter();
	painter.begin(this);
	
	painter.drawImage(0,0,this.img.scaled(this.width,this.height,Qt.KeepAspectRatio,Qt.SmoothTransformation));
	
	painter.end();
}

QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   var codec = QTextCodec.codecForName(new QByteArray("ISO-8859-1"));
   ts.setCodec(codec);
   return ts.readAll();
}

/* To use the application in standalone mode, remove the comment around this code
var wdgt = new ScrapBook();
wdgt.show();

QCoreApplication.exec();
*/
